<?php 
include_once('htmllib/lib/linux/fs.php');

class Updater
{
	const INIT_DIR        = '/etc/init.d/';
	const BRAND_NAME      = 'lxlabs';

	private $system_globals = NULL;
	private $program        = NULL;
	private $program_home   = NULL;
	private $root_path      = NULL;
	private $etc_path       = NULL;
	private $php_path       = NULL;
	private $html_base      = NULL;
	private $httpd_root     = NULL;
	private $file_core      = NULL;
	private $program_init   = NULL;

	public function __construct()
	{
		$this->system_globals = new Sgbl();

		$this->program        =	$this->system_globals->getProductName();
		$this->program_home   = $this->system_globals->getProgramHome();
		$this->root_path      = $this->system_globals->getRoot();
		$this->etc_path       = $this->system_globals->getEtc();
		$this->php_path       = $this->system_globals->getPHPPath();
		$this->html_base      = $this->system_globals->getHTMLBase();
		$this->httpd_root     = $this->system_globals->getHTMLBase();
		$this->file_core      = $this->html_base . '/htmllib/filecore/';
		$this->program_init   = $this->file_core . 'init.program';
	}

	public function restartInitProgram($program)
	{
		// We just need to kill the main server, and leave the wrapper alone.
		exec_with_all_closed('/etc/init.d/' . $program . ' lxrestart');
	}

	public function initCleanup($type)
	{
		echo 'This will take a long time, please wait ...' . PHP_EOL;
		system($this->php_path . ' ../bin/common/tmpupdatecleanup.php --type=' . $type);
	}

	public function createMainPaths()
	{
		lxfile_mkdir($this->etc_path . '/conf');
		lxfile_mkdir($this->root_path . '/pid');
		lxfile_mkdir($this->root_path . '/log');
		lxfile_mkdir($this->httpd_root);
	}

	public function fixLxlabsPermissions()
	{
		lxfile_mkdir($this->root_path . '/session');
		lxfile_unix_chown_rec($this->root_path,                    self::BRAND_NAME);
		lxfile_unix_chmod_rec($this->root_path . '/sbin/',         '0755');
		lxfile_unix_chmod_rec($this->root_path . '/httpdocs/img/', '0755');
		lxfile_unix_chmod($this->etc_path,                         '0700');
		lxfile_unix_chmod($this->root_path . '/log',               '0700');
		lxfile_unix_chmod($this->root_path . '/session',           '0700');
		lxfile_symlink($this->php_path,                            '/usr/bin/lphp.exe');
	}

	public function createProgramService($program)
	{
		echo 'Creating ' . $program . ' entry on init.d' . PHP_EOL;

		if (lxfile_exists($this->program_init))
		{
			echo 'Overwriting the file on init.d' . PHP_EOL;
			lxfile_cp($this->program_init, self::INIT_DIR . $program);
		}
		else
		{
			lxfile_cp($this->program_init, self::INIT_DIR . $program);
		}

		lxfile_unix_chmod(self::INIT_DIR . $program, '0755');
	}

	/**
	 * @todo rewrite /usr/local/lxlabs/ folder to /usr/local/quijost
	 * Enter description here ...
	 */
	public function copyPHPini()
	{
		echo 'Coping initial php.ini' . PHP_EOL;
		lxfile_cp($this->file_core . 'php.ini', '/usr/local/lxlabs/ext/php/etc/php.ini');
	}

	public function installCore($path, $file, $version)
	{
		echo 'Core installing with version ' . $version . PHP_EOL;

		$program_cache = '/var/cache/' . $this->program;
		$file_version = $file . '.' . $version . '.zip';
		$cached_file = $program_cache . '/' . $file_version;

		lxfile_mkdir($program_cache);

		if (!lxfile_real($program_cache. '/' . $file_version))
		{
			echo 'Downloading download.quijost.com/download/' . $file_version . PHP_EOL;

			$result = system('cd ' .$program_cache .'; rm -f ' . $file . '*.zip; wget download.quijost.com/download/' . $file_version);
			$count = 0;
			$uncompress = TRUE;

			while (lxshell_return('unzip', '-t', $cached_file))
			{
				sleep(1);
				$count++;
				echo 'Waiting ' . $count . ' seconds' . PHP_EOL;

				if($count > 5)
				{
					echo 'Error downloading file download.quijost.com/download/' . $file_version . PHP_EOL;
					$uncompress = FALSE;
					break;
				}
					
				if($uncompress)
				{
					system('cd ' . $path . '; unzip -oq ' . $cached_file);
				}
			}
		}
		else
		{
			echo 'File detected on cache: ' . $cached_file . PHP_EOL;
		}
	}

	public function downloadThirparty()
	{

		echo 'Downloading thirparty software...' . PHP_EOL;

		$string = @file_get_contents('http://download.quijost.com/download/thirdparty/' . $this->program . '-version.list');

		if(!empty($string))
		{
			$path = '/usr/local/lxlabs/' . $this->program . '/';
			$file = $this->program . '-thirdparty';
				
			$string = str_replace("\n", '', trim($string));
			$version = str_replace("\r", '', $string);
				
			$this->installCore($path, $file, $version);
				
			$this->checkPhpMyAdminConfig();
		}
		else
		{
			echo 'Error: no version found on repository. Please report.' . PHP_EOL;
		}
	}

	public function checkPhpMyAdminConfig()
	{
		echo 'Checking permissions of PhpMyAdmin config ...' . PHP_EOL;

		$phpmyadmin_config = '/usr/local/lxlabs/' . $this->program . '/httpdocs/thirdparty/phpMyAdmin/config.inc.php';
			
		if(lxfile_exists($phpmyadmin_config))
		{
			lxfile_unix_chmod($phpmyadmin_config, '0644');
		}
		else
		{
			echo 'PhpMyAdmin config file not found. Skipping.' . PHP_EOL;
		}
	}

	public function prepareDefaultDir()
	{
		echo 'Preparing the default directory ...' . PHP_EOL;
		lxfile_mkdir($this->httpd_root . '/default/');

		echo 'Copying skeleton structure ...' . PHP_EOL;
		lxfile_cp("../file/skeleton.zip", $this->httpd_root . '/skeleton.zip');

		echo 'Uncompressing skeleton structure ...' . PHP_EOL;
		lxshell_unzip("__system__",  $this->httpd_root . '/default/', '../file/skeleton.zip');

		echo 'Copying default index page ...' . PHP_EOL;
		lxfile_cp("../file/default_index.html", $this->httpd_root . '/default/index.html');

		echo 'Copying default disable page ...' . PHP_EOL;
		lxfile_mkdir($this->httpd_root . '/disable/');
		lxfile_cp("../file/disable.html", $this->httpd_root . '/disable/index.html');
	}

	public function installPHPgd()
	{
		echo 'Checking if exist PHP gd module ...' . PHP_EOL;

		//Return 0 means NO no problem finding the version and 1 is problem because the package is not found
		$detect_result = lxshell_return('rpm', '-q', 'php-gd1');

		// If not result like "php-gd-5.2.17-1", then install
		if($detect_result === 0)
		{
			echo 'Installing PHP 1 gd module ...' . PHP_EOL;
			$install_result = lxshell_return('yum', '-y', 'install', 'php-gd');
		}
	}

	public function setLxsuexec()
	{
		echo 'Removing and copying lxsuexec' . PHP_EOL;

		$lxsuexec_binary = '/usr/bin/lxsuexec';

		if(lxfile_exists($lxsuexec_binary))
		{
			lxfile_rm($lxsuexec_binary);
		}

		lxfile_cp('../cexe/lxsuexec', '/usr/bin');
		lxshell_return('chmod', '755', $lxsuexec_binary);
		lxshell_return('chmod', 'ug+s', $lxsuexec_binary);
	}

	public function setLxexec()
	{
		echo 'Removing and copying lxexec' . PHP_EOL;

		$lxexec_binary = '/usr/bin/lxexec';

		if(lxfile_exists($lxexec_binary))
		{
			lxfile_rm($lxexec_binary);
		}

		lxfile_cp('../cexe/lxexec', '/usr/bin');
		lxshell_return('chmod', '755', $lxexec_binary);
	}

	public function setLxphpsu()
	{
		echo 'Setting permissions lxphpsu' . PHP_EOL;

		lxfile_unix_chown('../cexe/lxphpsu', 'root:root');
		lxfile_unix_chmod('../cexe/lxphpsu', '0755');
		lxfile_unix_chmod('../cexe/lxphpsu', 'ug+s');

		lxfile_unix_chmod('../file/phpsuexec.sh', '0755');
	}

	public function installBogofilter()
	{
		$database = '/wordlist.db';
		$bogofilter_path = '/var/bogofilter';
		$wordlist_database = $bogofilter_path . $database;
		$program_wordlist_database = $bogofilter_path . '/' . $this->program . $database;

		if (!lxfile_exists($program_wordlist_database))
		{
			echo 'Installing bogofilter wordlist database' . PHP_EOL;

			// Create the needed dirs
			lxfile_mkdir($bogofilter_path);
			lxfile_mkdir($program_wordlist_database);

			if(lxfile_exists($wordlist_database))
			{
				lxfile_rm($wordlist_database);
			}

			$data = file_get_contents('http://download.quijost.com/download/' . $database);
			file_put_contents($wordlist_database, $data);

			$popuser = 'lxpopuser';
			lxfile_unix_chown_rec($bogofilter_path, $popuser . ':' .  $popuser);
			lxfile_cp($wordlist_database, $program_wordlist_database);
		}
	}

	public function migrateClientsDir()
	{
		if(!lxfile_exists($this->program_home . '/client'))
		{
			echo 'Migrating clients directory to client' . PHP_EOL;
			lxfile_mv_rec($this->program_home . '/clients', $this->program_home . '/client');
		}
	}

	public function installAwstats($ver)
	{
		if (!is_numeric($ver)) {
			return;
		}

		echo 'Installing AWstats ...' . PHP_EOL;

		lxfile_mkdir('/var/cache/kloxo');
		lxfile_mkdir('/home/kloxo/httpd/awstats/');

		if (!lxfile_real('/var/cache/kloxo/lxawstats' . $ver . '.tar.gz'))
		{
			echo 'Downloading awstats file' . PHP_EOL;
			system('cd /var/cache/kloxo/ ; rm -f lxawstats*.tar.gz;');
			lxshell_return('wget', '-c', 'download.quijost.com/download/lxawstats' . $ver . '.tar.gz', '-O', '/var/cache/kloxo/lxawstats' . $ver . '.tar.gz');
		}
		else
		{
			echo 'File detected on cache' . PHP_EOL;
		}

		lxfile_rm_rec('/home/kloxo/httpd/awstats/tools/');
		lxfile_rm_rec('/home/kloxo/httpd/awstats/wwwroot/');

		echo 'Uncompressing awstats file' . PHP_EOL;
		system("cd /home/kloxo/httpd/awstats/ ; tar -xzf /var/cache/kloxo/lxawstats$ver.tar.gz tools wwwroot docs");
	}

	public function configureLxrestart()
	{
		echo 'Configuring lxrestart binary' . PHP_EOL;

		system('cp ../cexe/lxrestart /usr/sbin/');
		system('chown root:root /usr/sbin/lxrestart');
		system('chmod 755 /usr/sbin/lxrestart');
		system('chmod ug+s /usr/sbin/lxrestart');
	}
	
	public function createMainDatabase($type = 'master', $root_user = NULL, $root_password = NULL)
    {
        global $gbl, $sgbl, $login, $ghtml;
        
        switch($this->system_globals->getDataBaseType())
        {
            case 'mysql':
                $req = @mysql_connect('localhost', $root_user, $root_password);
            break;
            default:
                $req = new PDO('sqlite:' . $this->system_globals->getDataBaseFile());
            break;
        }
        
        if(!$req) 
        {
            echo 'Error: Could not Connect to Database on localhost using root user' . PHP_EOL;
            die();
        }
        else 
        {
            //$sqlcm = lfile_get_contents("__path_program_root/httpdocs/sql/init/$type.sql");
        
            $dbname = $this->system_globals->getDataBaseFile();
            $pguser = $this->system_globals->getAdminUser();
            $admin_file_pass = $this->system_globals->getAdminFilePass();
        
            $database_password = randomString(9);
            $dbadminpass = client::createDbPass($database_password);
            
            switch($sgbl->getDataBaseType())
            {
                case 'mysql':
                    echo 'Creating database ' . $dbname . ' with user ' . $pguser. PHP_EOL;
                    mysql_query('CREATE DATABASE ' . $dbname);
                    echo 'Granting privileges to user ' . $pguser. PHP_EOL;
                    mysql_query("GRANT ALL ON $dbname.* TO '$pguser'@'localhost' identified by '$dbadminpass';");
                break;
            }
        
            echo 'Writing password config file on ' . $admin_file_pass . PHP_EOL;
            lfile_put_contents($admin_file_pass, $dbadminpass);
            lxfile_generic_chown($admin_file_pass, 'lxlabs');
        }
    }
    
    public function createGeneral()
    {
        global $sgbl, $login, $ghtml; 
        
        $general = new General(null, null, 'admin');
        $general->initThisDef();
        $g = $general->generalmisc_b;
    
        $g->npercentage = 90;
        $g->dpercentage = 110;
        $g->attempts = 3;
        $g->loginhistory_time = 2;
        $g->traffichistory_time = 6;
    
        $list = array('Billing', 'Complaint', 'Other');
        $h = array();
        foreach($list as $l)
        {
            $cat = new helpdeskcategory_a(NULL, NULL, $l);
            $h[$l] = $cat;
        }
        $general->helpdeskcategory_a = $h;
    
        $general->dbaction = 'add';
        $general->write();
    }
}

function os_isSelfSystemUser()
{
    return (posix_getuid() === 0);
}

function os_isSelfSystemOrLxlabsUser()
{
    $pwd = posix_getpwuid(posix_getuid());
    
	return os_isSelfSystemUser() || $pwd['name'] === 'lxlabs';
}

function os_getUptime()
{
    return trim(strtilfirst(trim(file_get_contents('/proc/uptime')), ' '));
}

function os_getLoadAvg($flag = false)
{
    $vv = explode(' ', trimSpaces(trim(file_get_contents('/proc/loadavg'))));
    if ($flag) { return $vv[2]; }

    return  "{$vv[0]} {$vv[1]} {$vv[2]}";
}

function os_fix_fstab()
{
    $list = lfile_trim('/etc/fstab');

    foreach($list as $l)
    {
        $nl = trimSpaces($l);

        $n = explode(' ', $nl);

        if ($n[1] !== '/home' && $n[1] !== '/')
        {
            $out[] = $l;
            continue;
        }
        if (!csa($n[3], 'usrquota'))
        {
            $mount[] = $n[1];
            $n[3] = "$n[3],usrquota,grpquota";
        }
        $o = implode("\t", $n);
        $out[] = $o;
    }

    $out = implode("\n", $out);
    $out .= "\n";
    dprint($out);
    lfile_put_contents('/etc/fstab', $out);
    dprintr($mount);
    foreach($mount as $m) {
        system("mount $m -o remount");
    }

}

function os_set_quota($username, $disk)
{
    if (!$username) return;

    if (!$disk) $disk = 0;

    $inode = $disk * 500;
    lxshell_return("setquota", "-u", $username, $disk, $disk, $inode, $inode, "-a");
}

function os_createUserQuota()
{
    os_fix_fstab();
    system("quotacheck -afmuvg");
    system("quotaon -auvg");
}

function os_get_home_dir($user)
{
    $pwd = posix_getpwnam($user);
    return $pwd['dir'];
}

function os_get_allips()
{
    $out = lxshell_output('ifconfig');
    $list = explode("\n", $out);
    foreach($list as $l) {
        $l = trim($l);
        if (!csa($l, 'inet addr:')) {
            continue;
        }
        $ip = strfrom($l, 'inet addr:');
        $ip = strtilfirst($ip, " ");
        if (csb($ip, '127.0')) {
            continue;
        }
        $iplist[] = $ip;
    }
    return $iplist;
}

function os_disable_user($username)
{
    lxshell_return('usermod', '-L', $username);
}

function os_enable_user($username)
{
    lxshell_return('usermod', '-U', $username);
}

function os_kill_process_user($username)
{
    $uid = os_get_uid_from_user($username);
    if($uid)
    {
        lxshell_return('pkill', '-u', $uid);
        lxshell_return('pkill', '-9', '-u', $uid);
    }
}

function os_create_default_slave_driver_db()
{
	echo 'Creating default slave driver database' . PHP_EOL;
    $a['web'] = 'apache';
    $a['dns'] = 'bind';
    $a['spam'] = 'spamassassin';
    slave_save_db('driver', $a);
}

function os_fix_lxlabs_permission()
{
    global $gbl, $sgbl, $login, $ghtml; 
    lxfile_mkdir($sgbl->getRoot() . '/session');
    lxfile_unix_chown_rec($sgbl->getRoot(), 'lxlabs');
    lxfile_unix_chmod_rec($sgbl->getRoot() . '/sbin/', '0755');
    lxfile_unix_chmod_rec($sgbl->getRoot() . '/httpdocs/img/', '0755');
    lxfile_unix_chmod($sgbl->getEtc(), '0700');
    lxfile_unix_chmod($sgbl->getRoot() . '/log', '0700');
    lxfile_unix_chmod($sgbl->getRoot() . '/session', '0700');
    lxfile_symlink($sgbl->getPHPPath(), '/usr/bin/lphp.exe');
}

function os_userdel($name)
{
    lxshell_return('userdel', '-r', $name);
}

function os_create_system_user($basename, $password, $id, $shell, $dir = '/tmp')
{
    dprint("In Create User $basename, $id $password $shell");
    $i = null;
    $name = $basename;
    while (true)
    {
        try {
            $ret = uuser__linux::checkIfUserExists($name, $id);
            if ($ret) {
                return $name;
            } else {
                break;
            }
        } catch (exception $e) {
            $i++;
            $name = "$basename$i";
        }
    }
    $ret = lxshell_return("useradd", "-m", "-c", uuser::getUserDescription($id), "-d", $dir, "-s", $shell, "-p", $password, $name);

    if($ret) throw new lxexception("could_not_create_user", '', $name);

    return $name;
}

function os_service_manage($serv, $act)
{
    exec_with_all_closed("/etc/init.d/$serv $act");
}

function os_is_arch_sixfour()
{
    if (!lxfile_exists('/proc/xen'))
    {
        $arch = trim(`arch`);
        return $arch === 'x86_64';
    }
    else
    {
        $q = lfile_get_contents('/etc/rpm/platform');
        if ($q === 'i686-redhat-linux') {
            return false;
        }
        return true;
    }
}

function os_is_php_six_four()
{
    $v = lxshell_output("rpm -q --queryformat '%{ARCH}' php");
    $v = trim($v);
    return ($v === 'x86_64');
}

function os_restart_program()
{
    global $gbl, $sgbl, $login, $ghtml; 
    // We just need to kill the main server, and leave the wrapper alone.
    exec_with_all_closed('/etc/init.d/' . $sgbl->getProductName() . ' lxrestart');
}

function os_get_network_gateway()
{
    $res = lxshell_output("route", "-n");
    $re = explode("\n", $res);

    foreach($re as $r) {
        if (csb($r, "0.0.0.0")) {
            $r = trimSpaces($r);
            $o = explode(" ", $r);
            return trim($o[1]);
        }
    }
    return null;
}

function os_get_default_ethernet()
{
    $res = lxshell_output("route", "-n");
    $re = explode("\n", $res);

    foreach($re as $r) {
        if (csb($r, "0.0.0.0")) {
            $r = trimSpaces($r);
            $o = explode(" ", $r);
            return trim($o[7]);
        }
    }
    return null;
}

function os_getCpuNum()
{
    $list = lfile("/proc/cpuinfo");

    $i = 0;

    foreach($list as $l) {
        if (csa($l, "processor")) {
            $i++;
        }
    }
    return $i;
}

function os_get_user_from_uid($uid)
{
    $pwd = posix_getpwuid($uid);
    if ($pwd['name']) return $pwd['name'];

    return $uid;
}

function os_get_uid_from_user($user)
{
    $pwd = posix_getpwnam($user);

    if ($pwd['uid']) return $pwd['uid'];
    // If the user doesn't exist return a very large number.
    return 10000000;
}

function os_get_gid_from_user($user)
{
    $pwd = posix_getpwnam($user);

    if ($pwd['gid']) return $pwd['gid'];
    // If the user doesn't exist return a very large number.
    return 10000000;
}

function os_getpid()
{
    return posix_getpid();
}

// Please note that os_get_commandname actually returns the first argument (NOt the actual command), since the actuall command would be 'php'
function os_get_commandname($pid)
{
    if (!lxfile_exists("/proc/$pid")) return null;

    $cmd = lfile_get_contents("/proc/$pid/cmdline");
    $cmd = explode("\0", $cmd);
    return $cmd[1];
}

// If pcntl is enabled, then sigterm is automatically defined. Silll
@ define('SIGTERM', 15);
@ define('SIGKILL', 9);

function os_killpid_by_name($name)
{
    system("pkill -f $name");
}

function os_killpid($pid)
{
    if (!$pid) return;
    if (intval($pid) < 10) {
        return;
    }
    posix_kill($pid, SIGTERM);
    usleep(10000);
    posix_kill($pid, SIGKILL);
}

function os_set_path()
{
    global $gbl, $sgbl, $login, $ghtml; 
    putenv('PATH=/sbin/:/usr/sbin/:/bin/:/usr/bin:/usr/local/bin/:/usr/local/sbin:' .$sgbl->getRoot() . '/bin:' . $sgbl->getRoot() .'/sbin:');
}