<?php
/**
 * Singleton class file.
 *
 * Copyright (C) 2011 Quijost
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @copyright 2011, (c) Quijost
 * @author    Ángel Guzmán Maeso <angel@quijost.com>
 * @license   GNU Affero General Public License
 * @package   Core
 */
abstract class Singleton
{
	const UNDEFINED_CLASS = 1;
	
    protected static $__instances = array();
    
    # we don't permit an explicit call of the constructor! (like $v = new Singleton())
    protected function __construct() 
    { 
    }
    
	# we don't permit cloning the singleton (like $x = clone $v)
    protected function __clone() 
    { 
    }
    
    protected static function getInstance($class)
    {   
    	if(!self::checkExists($class)) 
    	{
    		if(class_exists($class))
    			self::$__instances[$class] = new $class();        
            else 
            	throw new Exception('Exception(Singleton): Cannot instantiate undefined class \'' . $class . '\'', self::UNDEFINED_CLASS);
		}
		
		return self::$__instances[$class];
    }
    
    #like you want to check for the existence of the class without necessarily creating it if it doesn't exit:
    public static function checkExists($class) 
    {
      	return isset(self::$__instances[$class]);
    }
}